<?php
/* Copyright (C) 2004-2017 Laurent Destailleur  <eldy@users.sourceforge.net>
 * Copyright (C) 2017 Mikael Carlavan <contact@mika-carl.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *  \file       htdocs/sitfac/admin/setup.php
 *  \ingroup    sitfac
 *  \brief      Admin page
 */


$res=@include("../../main.inc.php");                   // For root directory
if (! $res) $res=@include("../../../main.inc.php");    // For "custom" directory

// Libraries
require_once DOL_DOCUMENT_ROOT . "/core/lib/admin.lib.php";
require_once DOL_DOCUMENT_ROOT . "/core/lib/functions.lib.php";

dol_include_once("/sitfac/lib/sitfac.lib.php");

// Translations
$langs->load("sitfac@sitfac");
$langs->load("admin");

// Access control
if (! $user->admin) accessforbidden();

// Parameters
$action = GETPOST('action', 'alpha');
$value = GETPOST('value', 'alpha');

$reg = array();

/*
 * Actions
 */


include DOL_DOCUMENT_ROOT.'/core/actions_setmoduleoptions.inc.php';

$error=0;

// Action mise a jour ou ajout d'une constante
if ($action == 'update')
{
	$constname=GETPOST('constname','alpha');
	$constvalue=(GETPOST('constvalue_'.$constname) ? GETPOST('constvalue_'.$constname) : GETPOST('constvalue'));

	$consttype=GETPOST('consttype','alpha');
	$constnote=GETPOST('constnote');
	$res = dolibarr_set_const($db,$constname,$constvalue,'chaine',0,$constnote,$conf->entity);

	if (! $res > 0) $error++;

	if (! $error)
	{
		setEventMessages($langs->trans("SetupSaved"), null, 'mesgs');
	}
	else
	{
		setEventMessages($langs->trans("Error"), null, 'errors');
	}
}

/*
 * View
 */

llxHeader('', $langs->trans('SitFacSetup'));

// Subheader
$linkback = '<a href="' . DOL_URL_ROOT . '/admin/modules.php">' . $langs->trans("BackToModuleList") . '</a>';

print load_fiche_titre($langs->trans('SitFacSetup'), $linkback);

// Configuration header
$head = sitfac_prepare_admin_head();
dol_fiche_head(
	$head,
	'settings',
	$langs->trans("ModuleSitFacName"),
	0,
	"sitfac@sitfac"
);

$form = new Form($db);


// Setup page goes here
echo $langs->trans("SitFacSetupPage");


print '<table class="noborder" width="100%">';
print '<tr class="liste_titre">';
print '<td>'.$langs->trans("Description").'</td>';
print '<td colspan="2">';
$text = $langs->trans("Value");
print $form->textwithpicto($text, "", 1, 'help', '', 0, 2, 'idhelptext');
print '</td>';
print "</tr>\n";

print '<tr class="oddeven">';
print '<td>';
print $langs->trans('DescSITFAC_DISPLAY_PREVIOUS_PROGRESS');
print '</td>';
print '<td colspan="2">';
if ($conf->use_javascript_ajax)
{
	print ajax_constantonoff('SITFAC_DISPLAY_PREVIOUS_PROGRESS');
}
else
{
	if (empty($conf->global->SITFAC_DISPLAY_PREVIOUS_PROGRESS))
	{
		print '<a href="'.$_SERVER['PHP_SELF'].'?action=set_SITFAC_DISPLAY_PREVIOUS_PROGRESS">'.img_picto($langs->trans("Disabled"),'switch_off').'</a>';
	}
	else
	{
		print '<a href="'.$_SERVER['PHP_SELF'].'?action=del_SITFAC_DISPLAY_PREVIOUS_PROGRESS">'.img_picto($langs->trans("Enabled"),'switch_on').'</a>';
	}
}
print '</td>';
print '</tr>';

print '<tr class="oddeven">';
print '<td>';
print $langs->trans('DescSITFAC_DISPLAY_TOTAL_PROGRESS');
print '</td>';
print '<td colspan="2">';
if ($conf->use_javascript_ajax)
{
	print ajax_constantonoff('SITFAC_DISPLAY_TOTAL_PROGRESS');
}
else
{
	if (empty($conf->global->SITFAC_DISPLAY_TOTAL_PROGRESS))
	{
		print '<a href="'.$_SERVER['PHP_SELF'].'?action=set_SITFAC_DISPLAY_TOTAL_PROGRESS">'.img_picto($langs->trans("Disabled"),'switch_off').'</a>';
	}
	else
	{
		print '<a href="'.$_SERVER['PHP_SELF'].'?action=del_SITFAC_DISPLAY_TOTAL_PROGRESS">'.img_picto($langs->trans("Enabled"),'switch_on').'</a>';
	}
}
print '</td>';
print '</tr>';

print '</table>';


// Page end
dol_fiche_end();
llxFooter();
