<?php
/* Copyright (C) 2004-2017 Laurent Destailleur  <eldy@users.sourceforge.net>
 * Copyright (C) 2017 Mikael Carlavan <contact@mika-carl.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *  \file       htdocs/sitfac/class/actions_sitfac.class.php
 *  \ingroup    sitfac
 *  \brief      File of class to manage actions on propal
 */
require_once DOL_DOCUMENT_ROOT.'/core/class/commonobject.class.php';
require_once DOL_DOCUMENT_ROOT.'/core/class/html.form.class.php';
require_once DOL_DOCUMENT_ROOT.'/user/class/usergroup.class.php';
require_once DOL_DOCUMENT_ROOT.'/comm/action/class/actioncomm.class.php';
require_once DOL_DOCUMENT_ROOT.'/contact/class/contact.class.php';
require_once DOL_DOCUMENT_ROOT.'/societe/class/societe.class.php';
require_once DOL_DOCUMENT_ROOT.'/expedition/class/expedition.class.php';
require_once DOL_DOCUMENT_ROOT.'/categories/class/categorie.class.php';
/*
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);
*/

if (!empty($conf->soustotal->enabled)) {
    dol_include_once("/soustotal/class/soustotal.class.php");
}

class ActionsSitFac
{ 
	function defineColumnField($parameters, &$pdf, &$action, $hookmanager)
	{
		global $langs, $db, $mysoc, $conf, $user;


		$langs->load('sitfac@sitfac');

        $cols = $pdf->cols;
        $object = isset($parameters['object']) ? $parameters['object']: null;

        if ($object && $object->element == 'facture' && $pdf->situationinvoice) {
            // Add sitfac columns
            $rank = $cols['progress']['rank'];

            $new_cols = array();
            $new_cols['previous_progress'] = array(
                'rank' => $rank - 1,
                'width' => 20, // in mm
                'status' => !empty($conf->global->SITFAC_DISPLAY_PREVIOUS_PROGRESS),
                'title' => array(
                    'textkey' => 'PreviousProgressShort'
                ),
                'border-left' => true, // add left line separator
            );

            $new_cols['progress'] = $cols['progress'];
            $new_cols['progress']['title'] = array(
                'textkey' => 'ProgressShort'
            );

            $new_cols['total_progress'] = array(
                'rank' => $rank + 1,
                'width' => 20, // in mm
                'status' => !empty($conf->global->SITFAC_DISPLAY_TOTAL_PROGRESS),
                'title' => array(
                    'textkey' => 'TotalProgressShort'
                ),
                'border-left' => true, // add left line separator
            );


            $this->results = $new_cols;
        }

		return 0;
	}

    function printPDFLine($parameters, &$model, &$action, $hookmanager)
    {
        global $langs, $conf, $user;

        $langs->load('sitfac@sitfac');

        $object = isset($parameters['object']) ? $parameters['object']: null;
        $i = isset($parameters['i']) ? $parameters['i']: 0;
        $pdf = isset($parameters['pdf']) ? $parameters['pdf']: null;

        if ($object && $object->element == 'facture' && $model->situationinvoice) {

            $line = isset($object->lines[$i]) ? $object->lines[$i] : null;
            $outputlangs = isset($parameters['outputlangs']) ? $parameters['outputlangs']: 0;
            $hidedetails = isset($parameters['hidedetails']) ? $parameters['hidedetails']: 0;
            $curY = isset($parameters['curY']) ? $parameters['curY']: 0;
            $nexY = isset($parameters['nexY']) ? $parameters['nexY']: 0;

            $skip = false;
            if (!empty($conf->soustotal->enabled) && $line) {
                $skip = SousTotal::isTitleOrSubTotalOrFree($line);
            }

            if ($line && !$skip) {
                // Situation progress
                $previousprogress = 0;

                if (method_exists($line, 'get_prev_progress'))
                {
                    $previousprogress = $line->get_prev_progress($object->id);
                }
                $totalprogress = $line->situation_percent;
                $progress = $totalprogress - $previousprogress;

                if ($model->getColumnStatus('previous_progress')) {
                    $model->printStdColumnContent($pdf, $curY, 'previous_progress', $previousprogress . '%');
                    $nexY = max($pdf->GetY(), $nexY);
                }

                $model->printStdColumnContent($pdf, $curY, 'progress', $progress . '%');
                $nexY = max($pdf->GetY(), $nexY);

                // Situation progress
                if ($model->getColumnStatus('total_progress')) {
                    $model->printStdColumnContent($pdf, $curY, 'total_progress', $totalprogress . '%');
                    $nexY = max($pdf->GetY(), $nexY);
                }
            }
        }
    }

    /**
     * 	afterPDFCreation
     * 	@param		object			Linked object
     */
    function afterPDFCreation($parameters, &$model, &$action, $hookmanager)
    {
        global $conf, $user, $langs, $db, $pdf, $mysoc;


        $file = isset($parameters['file']) ? $parameters['file'] : null;
        $object = isset($parameters['object']) ? $parameters['object'] : null;
        $outputlangs = isset($parameters['outputlangs']) ? $parameters['outputlangs'] : null;

        if ($object && $object->element == 'facture' && $model->situationinvoice && !in_array($model->name, array('facture_situation', 'facture_situation2')))
        {
            $pdf = pdf_getInstance($model->format);
            $pdf->Open();
            $pdf->setPrintHeader(false);
            $pdf->setPrintFooter(false);

            // Load PDF file
            $pages = $pdf->setSourceFile($file);
            for ($p = 1; $p <= $pages; $p++)
            {
                $templateIdx = $pdf->ImportPage($p);
                $size = $pdf->getTemplatesize($templateIdx);

                $portrait = $size['h'] > $size['w'] ? true : false;

                $pdf->AddPage($portrait ? 'P' : 'L');
                $pdf->useTemplate($templateIdx);
            }

            $heightforfooter = $model->marge_basse + 8; // Height reserved to output the footer (value include bottom margin)
            if (!empty($conf->global->MAIN_GENERATE_DOCUMENTS_SHOW_FOOT_DETAILS)) {
                $heightforfooter += 6;
            }

            $pdf->AddPage('', '', true);
            $pdf->setPage($pages+1);
            $pdf->setTopMargin($model->marge_haute);
            $pdf->setPageOrientation('', 1, $heightforfooter); // The only function to edit the bottom margin of current page to set it.

            $default_font_size = pdf_getPDFFontSize($outputlangs);

            $pdf->SetFont('','', $default_font_size - 1);   // Into loop to work with multipage
            $pdf->SetTextColor(0,0,0);

            if (empty($conf->global->MAIN_PDF_DONOTREPEAT_HEAD)) {
                $this->_pagehead($pdf, $model, $object, 0, $outputlangs);
            }


            $tab_top_newpage = (empty($conf->global->MAIN_PDF_DONOTREPEAT_HEAD) ? 42 : 10);

            $nexY = $tab_top_newpage + 7;
            $curY = $nexY;
            $curX = $model->posxdesc-1;

            $previousinvoices = sizeof($object->tab_previous_situation_invoice) ? $object->tab_previous_situation_invoice : array();

            $remain_to_pay = 0;

            // Proposal total
            $propals = array();
            $orders = array();

            if (sizeof($previousinvoices))
            {
                foreach ($previousinvoices as $invoice)
                {
                    if ($invoice->is_first())
                    {
                        $invoice->fetchObjectLinked();

                        $propals = isset($invoice->linkedObjects['propal']) ? $invoice->linkedObjects['propal'] : array();
                        $orders = isset($invoice->linkedObjects['commande']) ? $invoice->linkedObjects['commande'] : array();
                    }
                }
            }
            else
            {
                if ($object->is_first())
                {
                    $object->fetchObjectLinked();

                    $propals = isset($object->linkedObjects['propal']) ? $object->linkedObjects['propal'] : array();
                    $orders = isset($object->linkedObjects['commande']) ? $object->linkedObjects['commande'] : array();

                }
            }

            $sign=1;
            if ($object->type == 2 && ! empty($conf->global->INVOICE_POSITIVE_CREDIT_NOTE)) $sign=-1;

            if (count($propals))
            {
                $propal = array_pop($propals);

                $total_ht = ($conf->multicurrency->enabled && $propal->mylticurrency_tx != 1 ? $propal->multicurrency_total_ht : $propal->total_ht);
                $remain_to_pay = $total_ht;

                $pdf->SetTextColor(0,0,60);
                $pdf->SetFont('','', $default_font_size - 1);


                $pdf->SetXY($model->marge_gauche, $tab_top_newpage + 1);

                $label = $outputlangs->transnoentities("ProposalTotalHT");
                $pdf->MultiCell($model->page_largeur-($model->marge_droite+$model->marge_gauche), 3, $label. ' '.price($sign * ($total_ht + (! empty($propal->remise)?$propal->remise:0))), 0, 'L', 0);

                $pdf->SetFont('','', $default_font_size - 1);

                // Output Rect
                $pdf->SetDrawColor(128,128,128);
                $model->printRect($pdf, $model->marge_gauche, $tab_top_newpage, $model->page_largeur-$model->marge_gauche-$model->marge_droite, 6);	// Rect prend une longueur en 3eme param et 4eme param

                $curY += 4;
            }
            else if (count($orders))
            {
                $order = array_pop($orders);

                $total_ht = ($conf->multicurrency->enabled && $order->mylticurrency_tx != 1 ? $order->multicurrency_total_ht : $order->total_ht);
                $remain_to_pay = $total_ht;
            }



            $outputlangs->load('orders');
            $outputlangs->load('propal');

            $posy = $curY;
            $posx = $curX;

            $width = 70;
            $width2 = $model->page_largeur - $posx - $width - $model->marge_droite;
            $useborder=0;
            $index = 0;

            $height = 4;

            $pdf->SetTextColor(0,0,60);
            $pdf->SetFont('','B', $default_font_size + 3);
            $pdf->SetXY($posx,$posy);

            $depositsamount = $object->getSumDepositsUsed();
            $deja_regle = $object->getSommePaiement();

            $tot_deja_regle = ($depositsamount + $deja_regle);

            $previousinvoices[] = $object;

            $force_to_zero = false;

            $idinv = 0;//count($previousinvoices);
            while ($idinv < count($previousinvoices))
            {
                $invoice = $previousinvoices[$idinv];

                $posy += 7;
                $index = 0;

                $pdf->SetTextColor(0,0,60);
                $pdf->SetFont('','B', $default_font_size + 3);

                $pageposbefore=$pdf->getPage();
                $pdf->startTransaction();

                $pdf->SetXY($posx,$posy);

                $ref = $outputlangs->transnoentities("InvoiceSituation").$outputlangs->convToOutputCharset(" n°".$invoice->situation_counter);

                if ($invoice->situation_final)
                {
                    $ref.= ' - DGD';
                    $force_to_zero = true;
                }

                $pdf->MultiCell($model->page_largeur-($model->marge_droite+$model->marge_gauche), 3, $ref. ' '.$invoice->ref. ' '. $outputlangs->transnoentities("InvoiceOfDate", dol_print_date($invoice->date,"%d/%m/%Y",false,$outputlangs)), 0, 'L', 0);

                $pdf->SetFont('','', $default_font_size - 1);

                $sign=1;
                if ($invoice->type == 2 && ! empty($conf->global->INVOICE_POSITIVE_CREDIT_NOTE)) $sign=-1;

                $posy += 7;
                // Total HT
                $pdf->SetFillColor(255,255,255);
                $pdf->SetXY($posx, $posy);
                $pdf->MultiCell($width, $height, $outputlangs->transnoentities("TotalHT"), 0, 'L', 1);

                $total_ht = ($conf->multicurrency->enabled && $invoice->mylticurrency_tx != 1 ? $invoice->multicurrency_total_ht : $invoice->total_ht);
                $pdf->SetXY($posx+$width, $posy);
                $pdf->MultiCell($width2, $height, price($sign * ($total_ht + (! empty($invoice->remise)?$invoice->remise:0)), 0, $outputlangs), 0, 'R', 1);

                $tvas = array();
                for ($i=0; $i<sizeof($invoice->lines); $i++)
                {
                    $prev_progress = $invoice->lines[$i]->get_prev_progress($invoice->id);
                    if ($prev_progress > 0 && !empty($invoice->lines[$i]->situation_percent)) // Compute progress from previous situation
                    {
                        if ($conf->multicurrency->enabled && $invoice->multicurrency_tx != 1) $tvaligne = $sign * $invoice->lines[$i]->multicurrency_total_tva * ($invoice->lines[$i]->situation_percent - $prev_progress) / $invoice->lines[$i]->situation_percent;
                        else $tvaligne = $sign * $invoice->lines[$i]->total_tva * ($invoice->lines[$i]->situation_percent - $prev_progress) / $invoice->lines[$i]->situation_percent;
                    } else {
                        if ($conf->multicurrency->enabled && $invoice->multicurrency_tx != 1) $tvaligne= $sign * $invoice->lines[$i]->multicurrency_total_tva;
                        else $tvaligne= $sign * $invoice->lines[$i]->total_tva;
                    }

                    if ($invoice->remise_percent) $tvaligne-=($tvaligne*$invoice->remise_percent)/100;

                    $tvaligne = price2num(price($tvaligne), 'MT');

                    $vatrate=(string) $invoice->lines[$i]->tva_tx;


                    if (($invoice->lines[$i]->info_bits & 0x01) == 0x01) $vatrate.='*';
                    if (! isset($tvas[$vatrate])) 				$tvas[$vatrate]=0;
                    $tvas[$vatrate] += $tvaligne;
                }


                // Show VAT by rates and total
                $pdf->SetFillColor(248,248,248);
                foreach($tvas as $tvakey => $tvaval)
                {
                    if ($tvakey != 0)    // On affiche pas taux 0
                    {
                        $index++;
                        $pdf->SetXY($posx, $posy + $height * $index);

                        $tvacompl='';
                        if (preg_match('/\*/',$tvakey))
                        {
                            $tvakey=str_replace('*','',$tvakey);
                            $tvacompl = " (".$outputlangs->transnoentities("NonPercuRecuperable").")";
                        }
                        $totalvat =$outputlangs->transcountrynoentities("TotalVAT",$mysoc->country_code).' ';
                        $totalvat.=vatrate($tvakey,1).$tvacompl;
                        $pdf->MultiCell($width, $height, $totalvat, 0, 'L', 1);

                        $pdf->SetXY($posx+$width, $posy + $height * $index);
                        $pdf->MultiCell($width2, $height, price($tvaval, 0, $outputlangs), 0, 'R', 1);
                    }
                }

                $index++;

                $total_ttc = ($conf->multicurrency->enabled && $invoice->multicurrency_tx != 1) ? $invoice->multicurrency_total_ttc : $invoice->total_ttc;

                $total_rg = 0;


                // Total TTC
                $pdf->SetXY($posx, $posy + $height * $index);
                $pdf->SetTextColor(0,0,60);
                $pdf->SetFillColor(224,224,224);
                $pdf->MultiCell($width, $height, $outputlangs->transnoentities("TotalTTC"), $useborder, 'L', 1);


                $pdf->SetXY($posx+$width, $posy + $height * $index);
                $pdf->MultiCell($width2, $height, price($sign * $total_ttc, 0, $outputlangs), $useborder, 'R', 1);


                if ($model->situationinvoice)
                {
                    $index++;
                    $date_infos = dol_getdate($object->date, true);
                    $date_rg = dol_mktime($date_infos['hours'], $date_infos['minutes'], $date_infos['seconds'], $date_infos['mon'], $date_infos['mday'], $date_infos['year']+1);


                    $pdf->SetXY($posx, $posy + $height * $index);
                    $pdf->SetTextColor(0,0,60);
                    $pdf->SetFillColor(241,241,241);
                    $pdf->MultiCell($width, $height, $outputlangs->transnoentities("ReductionWarrantyShort"), $useborder, 'L', 1);

                    $total_rg = price2num(price($total_ttc * 0.05), 'MT');


                    $pdf->SetXY($posx+$width, $posy + $height * $index);
                    $pdf->MultiCell($width2, $height, price(-$sign * $total_rg, 0, $outputlangs), $useborder, 'R', 1);

                    $total_with_rg_tc = $total_ttc - $total_rg;
                    $index++;

                    // Total TTC
                    $pdf->SetXY($posx, $posy + $height * $index);
                    $pdf->SetTextColor(0,0,60);
                    $pdf->SetFillColor(224,224,224);
                    $pdf->MultiCell($width, $height, $outputlangs->transnoentities("TotalWithRgTTC"), $useborder, 'L', 1);


                    $pdf->SetXY($posx+$width, $posy + $height * $index);
                    $pdf->MultiCell($width2, $height, price($sign * $total_with_rg_tc, 0, $outputlangs), $useborder, 'R', 1);
                }



                $index++;

                $pdf->SetTextColor(0,0,0);

                $creditnoteamount=$invoice->getSumCreditNotesUsed();
                $depositsamount=$invoice->getSumDepositsUsed();
                $deja_regle = $invoice->getSommePaiement();

                //print "x".$creditnoteamount."-".$depositsamount;exit;
                $resteapayer = price2num($invoice->total_ttc - $deja_regle - $total_rg - $creditnoteamount - $depositsamount, 'MT');
                if ($invoice->paye) $resteapayer=0;

                $y = 0;


                // Already paid + Deposits
                $tot_deja_regle += $deja_regle + $depositsamount;

                $pdf->SetXY($posx, $posy + $height * $index);
                $pdf->MultiCell($width, $height, $outputlangs->transnoentities("Paid"), 0, 'L', 0);
                $pdf->SetXY($posx+$width, $posy + $height * $index);
                $pdf->MultiCell($width2, $height, price($deja_regle + $depositsamount, 0, $outputlangs), 0, 'R', 0);

                // Credit note
                if ($creditnoteamount)
                {
                    $index++;
                    $pdf->SetXY($posx, $posy + $height * $index);
                    $pdf->MultiCell($width, $height, $outputlangs->transnoentities("CreditNotes"), 0, 'L', 0);
                    $pdf->SetXY($posx+$width, $posy + $height * $index);
                    $pdf->MultiCell($width2, $height, price($creditnoteamount, 0, $outputlangs), 0, 'R', 0);
                }

                // Escompte
                if ($invoice->close_code == Facture::CLOSECODE_DISCOUNTVAT)
                {
                    $index++;
                    $pdf->SetFillColor(255,255,255);

                    $pdf->SetXY($posx, $posy + $height * $index);
                    $pdf->MultiCell($width, $height, $outputlangs->transnoentities("EscompteOfferedShort"), $useborder, 'L', 1);
                    $pdf->SetXY($posx+$width, $posy + $height * $index);
                    $pdf->MultiCell($width2, $height, price($invoice->total_ttc - $deja_regle - $creditnoteamount - $depositsamount, 0, $outputlangs), $useborder, 'R', 1);

                    $resteapayer=0;
                }

                $index++;
                $pdf->SetTextColor(0,0,60);
                $pdf->SetFillColor(224,224,224);
                $pdf->SetXY($posx, $posy + $height * $index);
                $pdf->MultiCell($width, $height, $outputlangs->transnoentities("RemainderToPay"), $useborder, 'L', 1);
                $pdf->SetXY($posx+$width, $posy + $height * $index);
                $pdf->MultiCell($width2, $height, price($resteapayer, 0, $outputlangs), $useborder, 'R', 1);

                $pdf->SetFont('','', $default_font_size - 1);
                $pdf->SetTextColor(0,0,0);

                $index++;

                if ($deja_regle > 0)
                {
                    $title=$outputlangs->transnoentities("PaymentsAlreadyDone");
                    if ($invoice->type == 2) $title=$outputlangs->transnoentities("PaymentsBackAlreadyDone");

                    $pdf->SetFont('','', $default_font_size - 3);
                    $pdf->SetXY($posx, $posy + $height * $index);
                    $pdf->MultiCell($width, $height, $title, 0, 'L', 0);

                    //$pdf->line($tab3_posx, $tab3_top, $tab3_posx+$tab3_width, $tab3_top);

                    $index++;

                    $width4 = ($model->page_largeur - $model->marge_droite - $posx)/4;

                    $pdf->SetFont('','', $default_font_size - 4);
                    $pdf->SetXY($posx, $posy + $height * $index);
                    $pdf->MultiCell($width4, $height-1, $outputlangs->transnoentities("Payment"), 0, 'L', 0);
                    $pdf->SetXY($posx+$width4, $posy + $height * $index);
                    $pdf->MultiCell($width4, $height-1, $outputlangs->transnoentities("Amount"), 0, 'L', 0);
                    $pdf->SetXY($posx+$width4*2, $posy + $height * $index);
                    $pdf->MultiCell($width4, $height-1, $outputlangs->transnoentities("Type"), 0, 'L', 0);
                    $pdf->SetXY($posx+$width4*3, $posy + $height * $index);
                    $pdf->MultiCell($width4, $height-1, $outputlangs->transnoentities("Num"), 0, 'L', 0);

                    //$pdf->line($tab3_posx, $tab3_top-1+$tab3_height, $tab3_posx+$tab3_width, $tab3_top-1+$tab3_height);

                    $y=$height-1;

                    $pdf->SetFont('','', $default_font_size - 4);

                    $payments = $invoice->getListOfPayments();

                    if (sizeof($payments))
                    {
                        foreach ($payments as $payment)
                        {

                            $pdf->SetXY($posx, $posy + $height * $index + $y);
                            $pdf->MultiCell($width4, $height-1, dol_print_date($db->jdate($payment['date']),'day',false,$outputlangs,true), 0, 'L', 0);
                            $pdf->SetXY($posx+$width4, $posy + $height * $index + $y);
                            $pdf->MultiCell($width4, $height-1, price($sign * $payment['amount'], 0, $outputlangs), 0, 'L', 0);
                            $pdf->SetXY($posx+$width4*2, $posy + $height * $index + $y);
                            $oper = $outputlangs->transnoentitiesnoconv("PaymentTypeShort" . $payment['type']);

                            $pdf->MultiCell($width4, $height-1, $oper, 0, 'L', 0);
                            $pdf->SetXY($posx+$width4*3, $posy + $height * $index + $y);
                            $pdf->MultiCell($width4, $height-1, $payment['num'], 0, 'L', 0);

                            //$pdf->line($tab3_posx, $tab3_top+$y+3, $tab3_posx+$tab3_width, $tab3_top+$y+3);
                            $y+=($height-1);

                        }
                    }

                }



                // Output Rect
                $pdf->SetDrawColor(128,128,128);
                $model->printRect($pdf, $model->marge_gauche, $posy, $model->page_largeur-$model->marge_gauche-$model->marge_droite, $height * $index + $y);
                $posy += $height * $index + $y;

                $pageposafter=$pdf->getPage();
                if ($pageposafter > $pageposbefore)	// There is a pagebreak
                {
                    $pdf->rollbackTransaction(true);

                    $pageposafter=$pageposbefore;
                    $pdf->AddPage('','',true);
                    if (! empty($tplidx)) $pdf->useTemplate($tplidx);
                    if (empty($conf->global->MAIN_PDF_DONOTREPEAT_HEAD)) $this->_pagehead($pdf, $model, $object, 0, $outputlangs);
                    $pdf->setPage($pageposafter+1);
                    $pdf->setPageOrientation('', 1, 0);	// The only function to edit the bottom margin of current page to set it.

                    $posy = $tab_top_newpage + 1;

                }
                else
                {
                    $idinv++;
                    $remain_to_pay -= ($sign * ($total_ht + (! empty($invoice->remise) ? $invoice->remise : 0)));

                    $rem = 0;
                    if (count($invoice->lines)) {
                        foreach ($invoice->lines as $l) {
                            if ($l->fk_remise_except > 0) {
                                $discount = new DiscountAbsolute($db);
                                $result = $discount->fetch($l->fk_remise_except);
                                if ($result > 0) {
                                    $rem += $discount->amount_ht;
                                }
                            }

                        }
                    }

                    $remain_to_pay -= $rem;

                    $pdf->commitTransaction();
                }
            }

            if ($force_to_zero)
            {
                $remain_to_pay = 0;
            }

            $posy+=10;

            $pdf->setPageOrientation('', 1, 0);	// The only function to edit the bottom margin of current page to set it.


            $pdf->SetTextColor(0,0,60);
            $pdf->SetFont('','', $default_font_size - 1);
            $pdf->SetXY($model->marge_gauche, $posy + 1);
            $label = $outputlangs->transnoentities("RemainToBillHT");
            $pdf->MultiCell($model->page_largeur-($model->marge_droite+$model->marge_gauche), 3, $label. ' '.price($remain_to_pay), 0, 'L', 0);
            $pdf->SetDrawColor(128,128,128);
            $model->printRect($pdf, $model->marge_gauche, $posy, $model->page_largeur-$model->marge_gauche-$model->marge_droite, 7);


            if (method_exists($pdf,'AliasNbPages')) $pdf->AliasNbPages();

            $pdf->Close();

            $pdf->Output($file,'F');
        }

        return 0;
    }

    // phpcs:disable PEAR.NamingConventions.ValidFunctionName.PublicUnderscore
    /**
     *  Show top header of page.
     *
     *  @param	TCPDF		$pdf     		Object PDF
     *  @param  Facture		$object     	Object to show
     *  @param  int	    	$showaddress    0=no, 1=yes
     *  @param  Translate	$outputlangs	Object lang for output
     *  @param  Translate	$outputlangsbis	Object lang for output bis
     *  @return	void
     */
    protected function _pagehead(&$pdf, $model, $object, $showaddress, $outputlangs, $outputlangsbis = null)
    {
        global $conf, $langs, $db;

        $ltrdirection = 'L';
        if ($outputlangs->trans("DIRECTION") == 'rtl') $ltrdirection = 'R';

        // Load traductions files required by page
        $outputlangs->loadLangs(array("main", "bills", "propal", "companies"));

        $default_font_size = pdf_getPDFFontSize($outputlangs);

        pdf_pagehead($pdf, $outputlangs, $model->page_hauteur);

        // Show Draft Watermark
        if ($object->statut == $object::STATUS_DRAFT && (!empty($conf->global->FACTURE_DRAFT_WATERMARK))) {
            pdf_watermark($pdf, $outputlangs, $model->page_hauteur, $model->page_largeur, 'mm', $conf->global->FACTURE_DRAFT_WATERMARK);
        }

        $pdf->SetTextColor(0, 0, 60);
        $pdf->SetFont('', 'B', $default_font_size + 3);

        $w = 110;

        $posy = $model->marge_haute;
        $posx = $model->page_largeur - $model->marge_droite - $w;

        $pdf->SetXY($model->marge_gauche, $posy);

        // Logo
        if (empty($conf->global->PDF_DISABLE_MYCOMPANY_LOGO)) {
            if ($model->emetteur->logo) {
                $logodir = $conf->mycompany->dir_output;
                if (!empty($conf->mycompany->multidir_output[$object->entity])) {
                    $logodir = $conf->mycompany->multidir_output[$object->entity];
                }
                if (empty($conf->global->MAIN_PDF_USE_LARGE_LOGO)) {
                    $logo = $logodir.'/logos/thumbs/'.$model->emetteur->logo_small;
                } else {
                    $logo = $logodir.'/logos/'.$model->emetteur->logo;
                }
                if (is_readable($logo)) {
                    $height = pdf_getHeightForLogo($logo);
                    $pdf->Image($logo, $model->marge_gauche, $posy, 0, $height); // width=0 (auto)
                } else {
                    $pdf->SetTextColor(200, 0, 0);
                    $pdf->SetFont('', 'B', $default_font_size - 2);
                    $pdf->MultiCell($w, 3, $outputlangs->transnoentities("ErrorLogoFileNotFound", $logo), 0, 'L');
                    $pdf->MultiCell($w, 3, $outputlangs->transnoentities("ErrorGoToGlobalSetup"), 0, 'L');
                }
            } else {
                $text = $model->emetteur->name;
                $pdf->MultiCell($w, 4, $outputlangs->convToOutputCharset($text), 0, $ltrdirection);
            }
        }

        $pdf->SetFont('', 'B', $default_font_size + 3);
        $pdf->SetXY($posx, $posy);
        $pdf->SetTextColor(0, 0, 60);
        $title = $outputlangs->transnoentities("PdfInvoiceTitle");
        if ($object->type == 1) {
            $title = $outputlangs->transnoentities("InvoiceReplacement");
        }
        if ($object->type == 2) {
            $title = $outputlangs->transnoentities("InvoiceAvoir");
        }
        if ($object->type == 3) {
            $title = $outputlangs->transnoentities("InvoiceDeposit");
        }
        if ($object->type == 4) {
            $title = $outputlangs->transnoentities("InvoiceProForma");
        }
        if ($model->situationinvoice) {
            $title = $outputlangs->transnoentities("PDFInvoiceSituation");
        }
        if (!empty($conf->global->PDF_USE_ALSO_LANGUAGE_CODE) && is_object($outputlangsbis)) {
            $title .= ' - ';
            if ($object->type == 0) {
                if ($model->situationinvoice) {
                    $title .= $outputlangsbis->transnoentities("PDFInvoiceSituation");
                }
                $title .= $outputlangsbis->transnoentities("PdfInvoiceTitle");
            } elseif ($object->type == 1) {
                $title .= $outputlangsbis->transnoentities("InvoiceReplacement");
            } elseif ($object->type == 2) {
                $title .= $outputlangsbis->transnoentities("InvoiceAvoir");
            } elseif ($object->type == 3) {
                $title .= $outputlangsbis->transnoentities("InvoiceDeposit");
            } elseif ($object->type == 4) {
                $title .= $outputlangsbis->transnoentities("InvoiceProForma");
            }
        }
        $pdf->MultiCell($w, 3, $title, '', 'R');

        $pdf->SetFont('', 'B', $default_font_size);

        $posy += 5;
        $pdf->SetXY($posx, $posy);
        $pdf->SetTextColor(0, 0, 60);
        $textref = $outputlangs->transnoentities("Ref")." : ".$outputlangs->convToOutputCharset($object->ref);
        if ($object->statut == $object::STATUS_DRAFT) {
            $pdf->SetTextColor(128, 0, 0);
            $textref .= ' - '.$outputlangs->transnoentities("NotValidated");
        }
        $pdf->MultiCell($w, 4, $textref, '', 'R');

        $posy += 1;
        $pdf->SetFont('', '', $default_font_size - 2);

        if ($object->ref_client) {
            $posy += 4;
            $pdf->SetXY($posx, $posy);
            $pdf->SetTextColor(0, 0, 60);
            $pdf->MultiCell($w, 3, $outputlangs->transnoentities("RefCustomer")." : ".$outputlangs->convToOutputCharset($object->ref_client), '', 'R');
        }

        if (!empty($conf->global->PDF_SHOW_PROJECT_TITLE)) {
            $object->fetch_projet();
            if (!empty($object->project->ref)) {
                $posy += 3;
                $pdf->SetXY($posx, $posy);
                $pdf->SetTextColor(0, 0, 60);
                $pdf->MultiCell($w, 3, $outputlangs->transnoentities("Project")." : ".(empty($object->project->title) ? '' : $object->project->title), '', 'R');
            }
        }

        if (!empty($conf->global->PDF_SHOW_PROJECT)) {
            $object->fetch_projet();
            if (!empty($object->project->ref)) {
                $outputlangs->load("projects");
                $posy += 3;
                $pdf->SetXY($posx, $posy);
                $pdf->SetTextColor(0, 0, 60);
                $pdf->MultiCell($w, 3, $outputlangs->transnoentities("RefProject")." : ".(empty($object->project->ref) ? '' : $object->project->ref), '', 'R');
            }
        }

        $objectidnext = $object->getIdReplacingInvoice('validated');
        if ($object->type == 0 && $objectidnext) {
            $objectreplacing = new Facture($db);
            $objectreplacing->fetch($objectidnext);

            $posy += 3;
            $pdf->SetXY($posx, $posy);
            $pdf->SetTextColor(0, 0, 60);
            $pdf->MultiCell($w, 3, $outputlangs->transnoentities("ReplacementByInvoice").' : '.$outputlangs->convToOutputCharset($objectreplacing->ref), '', 'R');
        }
        if ($object->type == 1) {
            $objectreplaced = new Facture($db);
            $objectreplaced->fetch($object->fk_facture_source);

            $posy += 4;
            $pdf->SetXY($posx, $posy);
            $pdf->SetTextColor(0, 0, 60);
            $pdf->MultiCell($w, 3, $outputlangs->transnoentities("ReplacementInvoice").' : '.$outputlangs->convToOutputCharset($objectreplaced->ref), '', 'R');
        }
        if ($object->type == 2 && !empty($object->fk_facture_source)) {
            $objectreplaced = new Facture($db);
            $objectreplaced->fetch($object->fk_facture_source);

            $posy += 3;
            $pdf->SetXY($posx, $posy);
            $pdf->SetTextColor(0, 0, 60);
            $pdf->MultiCell($w, 3, $outputlangs->transnoentities("CorrectionInvoice").' : '.$outputlangs->convToOutputCharset($objectreplaced->ref), '', 'R');
        }

        $posy += 4;
        $pdf->SetXY($posx, $posy);
        $pdf->SetTextColor(0, 0, 60);

        $title = $outputlangs->transnoentities("DateInvoice");
        if (!empty($conf->global->PDF_USE_ALSO_LANGUAGE_CODE) && is_object($outputlangsbis)) {
            $title .= ' - '.$outputlangsbis->transnoentities("DateInvoice");
        }
        $pdf->MultiCell($w, 3, $title." : ".dol_print_date($object->date, "day", false, $outputlangs, true), '', 'R');

        if (!empty($conf->global->INVOICE_POINTOFTAX_DATE)) {
            $posy += 4;
            $pdf->SetXY($posx, $posy);
            $pdf->SetTextColor(0, 0, 60);
            $pdf->MultiCell($w, 3, $outputlangs->transnoentities("DatePointOfTax")." : ".dol_print_date($object->date_pointoftax, "day", false, $outputlangs), '', 'R');
        }

        if ($object->type != 2) {
            $posy += 3;
            $pdf->SetXY($posx, $posy);
            $pdf->SetTextColor(0, 0, 60);
            $title = $outputlangs->transnoentities("DateDue");
            if (!empty($conf->global->PDF_USE_ALSO_LANGUAGE_CODE) && is_object($outputlangsbis)) {
                $title .= ' - '.$outputlangsbis->transnoentities("DateDue");
            }
            $pdf->MultiCell($w, 3, $title." : ".dol_print_date($object->date_lim_reglement, "day", false, $outputlangs, true), '', 'R');
        }

        if ($object->thirdparty->code_client) {
            $posy += 3;
            $pdf->SetXY($posx, $posy);
            $pdf->SetTextColor(0, 0, 60);
            $pdf->MultiCell($w, 3, $outputlangs->transnoentities("CustomerCode")." : ".$outputlangs->transnoentities($object->thirdparty->code_client), '', 'R');
        }

        // Get contact
        if (!empty($conf->global->DOC_SHOW_FIRST_SALES_REP)) {
            $arrayidcontact = $object->getIdContact('internal', 'SALESREPFOLL');
            if (count($arrayidcontact) > 0) {
                $usertmp = new User($db);
                $usertmp->fetch($arrayidcontact[0]);
                $posy += 4;
                $pdf->SetXY($posx, $posy);
                $pdf->SetTextColor(0, 0, 60);
                $pdf->MultiCell($w, 3, $langs->transnoentities("SalesRepresentative")." : ".$usertmp->getFullName($langs), '', 'R');
            }
        }

        $posy += 1;

        $top_shift = 0;
        // Show list of linked objects
        $current_y = $pdf->getY();
        $posy = pdf_writeLinkedObjects($pdf, $object, $outputlangs, $posx, $posy, $w, 3, 'R', $default_font_size);
        if ($current_y < $pdf->getY()) {
            $top_shift = $pdf->getY() - $current_y;
        }

        if ($showaddress) {
            // Sender properties
            $carac_emetteur = pdf_build_address($outputlangs, $model->emetteur, $object->thirdparty, '', 0, 'source', $object);

            // Show sender
            $posy = !empty($conf->global->MAIN_PDF_USE_ISO_LOCATION) ? 40 : 42;
            $posy += $top_shift;
            $posx = $model->marge_gauche;
            if (!empty($conf->global->MAIN_INVERT_SENDER_RECIPIENT)) {
                $posx = $model->page_largeur - $model->marge_droite - 80;
            }

            $hautcadre = !empty($conf->global->MAIN_PDF_USE_ISO_LOCATION) ? 38 : 40;
            $widthrecbox = !empty($conf->global->MAIN_PDF_USE_ISO_LOCATION) ? 92 : 82;


            // Show sender frame
            $pdf->SetTextColor(0, 0, 0);
            $pdf->SetFont('', '', $default_font_size - 2);
            $pdf->SetXY($posx, $posy - 5);
            $pdf->MultiCell($widthrecbox, 5, $outputlangs->transnoentities("BillFrom"), 0, $ltrdirection);
            $pdf->SetXY($posx, $posy);
            $pdf->SetFillColor(230, 230, 230);
            $pdf->MultiCell($widthrecbox, $hautcadre, "", 0, 'R', 1);
            $pdf->SetTextColor(0, 0, 60);

            // Show sender name
            $pdf->SetXY($posx + 2, $posy + 3);
            $pdf->SetFont('', 'B', $default_font_size);
            $pdf->MultiCell($widthrecbox - 2, 4, $outputlangs->convToOutputCharset($model->emetteur->name), 0, $ltrdirection);
            $posy = $pdf->getY();

            // Show sender information
            $pdf->SetXY($posx + 2, $posy);
            $pdf->SetFont('', '', $default_font_size - 1);
            $pdf->MultiCell($widthrecbox - 2, 4, $carac_emetteur, 0, $ltrdirection);

            // If BILLING contact defined on invoice, we use it
            $usecontact = false;
            $arrayidcontact = $object->getIdContact('external', 'BILLING');
            if (count($arrayidcontact) > 0) {
                $usecontact = true;
                $result = $object->fetch_contact($arrayidcontact[0]);
            }

            // Recipient name
            if ($usecontact && ($object->contact->fk_soc != $object->thirdparty->id && (!isset($conf->global->MAIN_USE_COMPANY_NAME_OF_CONTACT) || !empty($conf->global->MAIN_USE_COMPANY_NAME_OF_CONTACT)))) {
                $thirdparty = $object->contact;
            } else {
                $thirdparty = $object->thirdparty;
            }

            $carac_client_name = pdfBuildThirdpartyName($thirdparty, $outputlangs);

            $mode =  'target';
            $carac_client = pdf_build_address($outputlangs, $model->emetteur, $object->thirdparty, ($usecontact ? $object->contact : ''), $usecontact, $mode, $object);

            // Show recipient
            $widthrecbox = !empty($conf->global->MAIN_PDF_USE_ISO_LOCATION) ? 92 : 100;
            if ($model->page_largeur < 210) {
                $widthrecbox = 84; // To work with US executive format
            }
            $posy = !empty($conf->global->MAIN_PDF_USE_ISO_LOCATION) ? 40 : 42;
            $posy += $top_shift;
            $posx = $model->page_largeur - $model->marge_droite - $widthrecbox;
            if (!empty($conf->global->MAIN_INVERT_SENDER_RECIPIENT)) {
                $posx = $model->marge_gauche;
            }

            // Show recipient frame
            $pdf->SetTextColor(0, 0, 0);
            $pdf->SetFont('', '', $default_font_size - 2);
            $pdf->SetXY($posx + 2, $posy - 5);
            $pdf->MultiCell($widthrecbox - 2, 5, $outputlangs->transnoentities("BillTo"), 0, $ltrdirection);
            $pdf->Rect($posx, $posy, $widthrecbox, $hautcadre);

            // Show recipient name
            $pdf->SetXY($posx + 2, $posy + 3);
            $pdf->SetFont('', 'B', $default_font_size);
            $pdf->MultiCell($widthrecbox - 2, 2, $carac_client_name, 0, $ltrdirection);

            $posy = $pdf->getY();

            // Show recipient information
            $pdf->SetFont('', '', $default_font_size - 1);
            $pdf->SetXY($posx + 2, $posy);
            $pdf->MultiCell($widthrecbox - 2, 4, $carac_client, 0, $ltrdirection);
        }

        $pdf->SetTextColor(0, 0, 0);
        return $top_shift;
    }

    // phpcs:disable PEAR.NamingConventions.ValidFunctionName.PublicUnderscore
    /**
     *   	Show footer of page. Need this->emetteur object
     *
     *   	@param	TCPDF		$pdf     			PDF
     * 		@param	Facture		$object				Object to show
     *      @param	Translate	$outputlangs		Object lang for output
     *      @param	int			$hidefreetext		1=Hide free text
     *      @return	int								Return height of bottom margin including footer text
     */
    protected function _pagefoot(&$pdf, $model, $object, $outputlangs, $hidefreetext = 0)
    {
        global $conf;
        $showdetails = empty($conf->global->MAIN_GENERATE_DOCUMENTS_SHOW_FOOT_DETAILS) ? 0 : $conf->global->MAIN_GENERATE_DOCUMENTS_SHOW_FOOT_DETAILS;
        return pdf_pagefoot($pdf, $outputlangs, 'INVOICE_FREE_TEXT', $model->emetteur, $model->marge_basse, $model->marge_gauche, $model->page_hauteur, $object, $showdetails, $hidefreetext);
    }

}


