<?php
/* Copyright (C) 2004-2017 Laurent Destailleur  <eldy@users.sourceforge.net>
 * Copyright (C) 2017 Mikael Carlavan <contact@mika-carl.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *	\file       htdocs/sitfac/lib/sitfac.lib.php
 *	\brief      Ensemble de fonctions de base pour le module sitfac
 * 	\ingroup	sitfac
 */


/**
 * Prepare array with list of tabs
 *
 * @param   Product	$object		Object related to tabs
 * @return  array				Array of tabs to show
 */
function sitfac_prepare_admin_head()
{
	global $db, $langs, $conf, $user;
	$langs->load("sitfac@sitfac");

	$h = 0;
	$head = array();
	
	$head[$h][0] = dol_buildpath("/sitfac/admin/setup.php", 1);
	$head[$h][1] = $langs->trans("Settings");
	$head[$h][2] = 'settings';
	$h++;
	
	$head[$h][0] = dol_buildpath("/sitfac/admin/about.php", 1);
	$head[$h][1] = $langs->trans("About");
	$head[$h][2] = 'about';
	$h++;

	return $head;
}

/**
 *	Return line total excluding tax
 *
 *	@param	Object		$object				Object
 *	@param	int			$i					Current line number
 *  @param  Translate	$outputlangs		Object langs for output
 *  @param	int			$hidedetails		Hide details (0=no, 1=yes, 2=just special lines)
 * 	@return	string							Return total of line excl tax
 */
function pdf_sitfac_getlinetotalexcltax($object,$i,$outputlangs,$hidedetails=0)
{
	global $conf, $hookmanager;

	$sign=1;
	if (isset($object->type) && $object->type == 2 && ! empty($conf->global->INVOICE_POSITIVE_CREDIT_NOTE)) $sign=-1;

	$reshook=0;
	$result='';
	//if (is_object($hookmanager) && ( (isset($object->lines[$i]->product_type) && $object->lines[$i]->product_type == 9 && ! empty($object->lines[$i]->special_code)) || ! empty($object->lines[$i]->fk_parent_line) ) )
	if (is_object($hookmanager))   // Old code is commented on preceding line. Reproduct this test in the pdf_xxx function if you don't want your hook to run
	{
		$special_code = $object->lines[$i]->special_code;
		if (! empty($object->lines[$i]->fk_parent_line)) $special_code = $object->getSpecialCode($object->lines[$i]->fk_parent_line);
		$parameters = array('i'=>$i,'outputlangs'=>$outputlangs,'hidedetails'=>$hidedetails,'special_code'=>$special_code, 'sign'=>$sign);
		$action='';
		$reshook = $hookmanager->executeHooks('pdf_getlinetotalexcltax',$parameters,$object,$action);    // Note that $action and $object may have been modified by some hooks

		if(!empty($hookmanager->resPrint)) $result.=$hookmanager->resPrint;
	}
    if (empty($reshook))
    {
	    if ($object->lines[$i]->special_code == 3)
    	{
    		return $outputlangs->transnoentities("Option");
    	}
        if (empty($hidedetails) || $hidedetails > 1)
        {
			$total_ht = ($conf->multicurrency->enabled && $object->multicurrency_tx != 1 ? $object->lines[$i]->multicurrency_total_ht : $object->lines[$i]->total_ht);

			$progress = 1;
			if ($object->lines[$i]->situation_percent > 0)
        	{
        		$prev_progress = 0;
        		
        		if (method_exists($object->lines[$i], 'get_prev_progress'))
        		{
					$prev_progress = $object->lines[$i]->get_prev_progress($object->id);
				}
				

				if ($prev_progress > 0)
				{
					$progress = ($object->lines[$i]->situation_percent - $prev_progress) / $object->lines[$i]->situation_percent;
				}
			}
			

			$result = price($sign * $total_ht * $progress, 0, $outputlangs);
		}
    }
	return $result;
}